@@ -29,17 +29,23 @@ type int_conv =
   | Int_X | Int_CX                 (*  %X | %#X        *)
   | Int_o | Int_Co                 (*  %o | %#o        *)
   | Int_u                          (*  %u              *)
+  | Int_Cd | Int_Ci | Int_Cu       (*  %#d | %#i | %#u *)
 
 (* Float conversion. *)
-type float_conv =
-  | Float_f | Float_pf | Float_sf  (*  %f | %+f | % f  *)
-  | Float_e | Float_pe | Float_se  (*  %e | %+e | % e  *)
-  | Float_E | Float_pE | Float_sE  (*  %E | %+E | % E  *)
-  | Float_g | Float_pg | Float_sg  (*  %g | %+g | % g  *)
-  | Float_G | Float_pG | Float_sG  (*  %G | %+G | % G  *)
-  | Float_F                        (*  %F              *)
-  | Float_h | Float_ph | Float_sh  (*  %h | %+h | % h  *)
-  | Float_H | Float_pH | Float_sH  (*  %H | %+H | % H  *)
+type float_flag_conv =
+  | Float_flag_                    (* %[feEgGFhH] *)
+  | Float_flag_p                   (* %+[feEgGFhH] *)
+  | Float_flag_s                   (* % [feEgGFhH] *)
+type float_kind_conv =
+  | Float_f                        (*  %f | %+f | % f  *)
+  | Float_e                        (*  %e | %+e | % e  *)
+  | Float_E                        (*  %E | %+E | % E  *)
+  | Float_g                        (*  %g | %+g | % g  *)
+  | Float_G                        (*  %G | %+G | % G  *)
+  | Float_F                        (*  %F | %+F | % F  *)
+  | Float_h                        (*  %h | %+h | % h  *)
+  | Float_H                        (*  %H | %+H | % H  *)
+type float_conv = float_flag_conv * float_kind_conv
 
 (***)
 
@@ -189,7 +195,7 @@ to transpose between related format types.
 NOTE [1]: the typing of Format_subst_ty requires not one format type, but
 two, one to establish the link between the format argument and the
 first six parameters, and the other for the link between the format
-argumant and the last six parameters.
+argument and the last six parameters.
 
 | Format_subst_ty :                                         (* %(...%) *)
     ('g, 'h, 'i, 'j, 'k, 'l,
@@ -214,7 +220,7 @@ function that proves that the relation is transitive
   -> ('a1, 'b1, 'c1, 'd1, 'e1, 'f1,
       'a3, 'b3, 'c3, 'd3, 'e3, 'f3) fmtty_rel
 
-does assume that the two input have exactly the same term structure
+does assume that the two inputs have exactly the same term structure
 (and is only every used for argument witnesses of the
 Format_subst_ty constructor).
 *)
@@ -232,7 +238,7 @@ type block_type =
                  when it leads to a new indentation of the current line *)
   | Pp_fits   (* Internal usage: when a block fits on a single line *)
 
-(* Formatting element used by the Format pretty-printter. *)
+(* Formatting element used by the Format pretty-printer. *)
 type formatting_lit =
   | Close_box                                           (* @]   *)
   | Close_tag                                           (* @}   *)
@@ -245,7 +251,7 @@ type formatting_lit =
   | Escaped_percent                                     (* @%%  *)
   | Scan_indic of char                                  (* @X   *)
 
-(* Formatting element used by the Format pretty-printter. *)
+(* Formatting element used by the Format pretty-printer. *)
 type ('a, 'b, 'c, 'd, 'e, 'f) formatting_gen =
   | Open_tag : ('a, 'b, 'c, 'd, 'e, 'f) format6 ->      (* @{   *)
     ('a, 'b, 'c, 'd, 'e, 'f) formatting_gen
@@ -385,13 +391,13 @@ and ('a, 'b, 'c, 'd, 'e, 'f) fmt =
       int_conv * ('x, 'y) padding * ('y, int64 -> 'a) precision *
       ('a, 'b, 'c, 'd, 'e, 'f) fmt ->
         ('x, 'b, 'c, 'd, 'e, 'f) fmt
-  | Float :                                                  (* %[feEgGF] *)
+  | Float :                                                  (* %[feEgGFhH] *)
       float_conv * ('x, 'y) padding * ('y, float -> 'a) precision *
       ('a, 'b, 'c, 'd, 'e, 'f) fmt ->
         ('x, 'b, 'c, 'd, 'e, 'f) fmt
   | Bool :                                                   (* %[bB] *)
-      ('a, 'b, 'c, 'd, 'e, 'f) fmt ->
-        (bool -> 'a, 'b, 'c, 'd, 'e, 'f) fmt
+      ('x, bool -> 'a) padding * ('a, 'b, 'c, 'd, 'e, 'f) fmt ->
+        ('x, 'b, 'c, 'd, 'e, 'f) fmt
   | Flush :                                                  (* %! *)
       ('a, 'b, 'c, 'd, 'e, 'f) fmt ->
         ('a, 'b, 'c, 'd, 'e, 'f) fmt
@@ -452,7 +458,7 @@ and ('a, 'b, 'c, 'd, 'e, 'f) fmt =
      We include a type Custom of "custom converters", where an
      arbitrary function can be used to convert one or more
      arguments. There is no syntax for custom converters, it is only
-     inteded for custom processors that wish to rely on the
+     intended for custom processors that wish to rely on the
      stdlib-defined format GADTs.
 
      For instance a pre-processor could choose to interpret strings
@@ -499,7 +505,7 @@ and ('a, 'b, 'c, 'd, 'e, 'f) ignored =
   | Ignored_float :                                          (* %_f *)
       pad_option * prec_option -> ('a, 'b, 'c, 'd, 'd, 'a) ignored
   | Ignored_bool :                                           (* %_B *)
-      ('a, 'b, 'c, 'd, 'd, 'a) ignored
+      pad_option -> ('a, 'b, 'c, 'd, 'd, 'a) ignored
   | Ignored_format_arg :                                     (* %_{...%} *)
       pad_option * ('g, 'h, 'i, 'j, 'k, 'l) fmtty ->
         ('a, 'b, 'c, 'd, 'd, 'a) ignored
@@ -540,7 +546,7 @@ let rec erase_rel : type a b c d e f g h i j k l .
     Bool_ty (erase_rel rest)
   | Format_arg_ty (ty, rest) ->
     Format_arg_ty (ty, erase_rel rest)
-  | Format_subst_ty (ty1, ty2, rest) ->
+  | Format_subst_ty (ty1, _ty2, rest) ->
     Format_subst_ty (ty1, ty1, erase_rel rest)
   | Alpha_ty rest ->
     Alpha_ty (erase_rel rest)
@@ -642,8 +648,8 @@ fun fmt1 fmt2 -> match fmt1 with
     Char (concat_fmt rest fmt2)
   | Caml_char rest ->
     Caml_char (concat_fmt rest fmt2)
-  | Bool rest ->
-    Bool (concat_fmt rest fmt2)
+  | Bool (pad, rest) ->
+    Bool (pad, concat_fmt rest fmt2)
   | Alpha rest ->
     Alpha (concat_fmt rest fmt2)
   | Theta rest ->
