@@ -20,7 +20,7 @@
   an error.
 *)
 
-val argv : string array
+external argv : string array = "%sys_argv"
 (** The command line arguments given to the process.
    The first element is the command name used to invoke the program.
    The following elements are the command-line arguments
@@ -145,9 +145,16 @@ val max_string_length : int
 (** Maximum length of strings and byte sequences. *)
 
 val max_array_length : int
-(** Maximum length of a normal array.  The maximum length of a float
-    array is [max_array_length/2] on 32-bit machines and
-    [max_array_length] on 64-bit machines. *)
+(** Maximum length of a normal array (i.e. any array whose elements are
+    not of type [float]). The maximum length of a [float array]
+    is [max_floatarray_length] if OCaml was configured with
+    [--enable-flat-float-array] and [max_array_length] if configured
+    with [--disable-flat-float-array]. *)
+
+val max_floatarray_length : int
+(** Maximum length of a floatarray. This is also the maximum length of
+    a [float array] when OCaml is configured with
+    [--enable-flat-float-array]. *)
 
 external runtime_variant : unit -> string = "caml_runtime_variant"
 (** Return the name of the runtime variant the program is running on.
